#!/usr/bin/python
# -*- coding: utf-8 -*-

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Author: Tomas "Tojaj" Mlcoch (xtojaj gmail dot com)
# Web: http://tojaj.com
#
# Copyright 2010 Tomas Mlcoch


"""Class for prints formatted results.
"""

import string
import os
import imp
import re

from operator import itemgetter

PREFIX = 'view/'


class ViewError(Exception):
    """Cannot load and use translation from file."""
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return repr(self.msg)


class ViewStdout(object):
    """Class for print formatted results to stdout."""
    langfile_re = re.compile(r'^lang_(.*)\.py$')

    def __init__(self):
        self.tr = {}


    def _ratings(self, movie):
        """Prepares string with ratings."""
        ratings = ''
        for item in movie:
            if item.endswith('Rating'):
                ratings += '{0:8} = {1:5}\n'.format(item[:-6], movie[item])
        return ratings.strip()


    def _add_sorting_value(self):
        """Calculates and add average film rating for movies."""
        for movie in self.movies:
            avrg_rating = 0
            count = 0
            for item in movie:
                if item.endswith('Rating'):
                    try:
                        avrg_rating += int(movie[item])
                        count += 1
                    except (ValueError, TypeError):
                        pass
            movie['AverageR'] = (count and (avrg_rating / count)) or 0


    def available_translation(self):
        """Returns list of available translations."""
        langlist = []
        for langfile in os.listdir(PREFIX):
            filematch = re.match(self.langfile_re, langfile)
            if filematch:
                langlist.append(filematch.group(1))
        return langlist


    def set_translation(self, lang='en'):
        """Set translation for output template."""
        tr_file = '{0}lang_{1}'.format(PREFIX, lang)
        mfile = None
        try:
            mfile, path, desc = imp.find_module(tr_file, None)
            self.tr = imp.load_module(tr_file, mfile, path, desc).TRANSLATION
        except (ImportError):
            raise ViewError('Cannot import translation.\n')
        finally:
            mfile and mfile.close()


    def print_out(self, list_of_movies):
        """Prepares movies in list and prints them on stdout."""
        self.movies = list_of_movies

        if not self.tr:
            try:
                self.set_translation()
            except (ImportError):
                pass

        try:
            template_raw = open(PREFIX+'template.tpl').read()
            template = string.Template(template_raw)
        except (IOError):
            raise ViewError('Cannot open template.')

        self._add_sorting_value()
        self.movies = sorted(self.movies, key=itemgetter('AverageR'))

        for movie in self.movies:
            subst_dict = {}
            subst_dict.update(self.tr)
            subst_dict.update(movie)
            if not 'MovieName' in movie and 'GuessedName' in movie:
                subst_dict['MovieName'] = movie['GuessedName']
            subst_dict['Ratings'] = self._ratings(movie)
            print template.safe_substitute(subst_dict)
        

if __name__ == '__main__':
    obj = ViewStdout()
